CON                       

  _clkmode = xtal1 + pll16x
  _clkfreq = 80_000_000


  ' Pin usage when connected through a Human Interface Board or Propeller Demo Board
  spiDI         = 0                            ' IO0 / DI / MOSI / CD pin
  spiDO         = 1                            ' IO1 / DO / MISO pin
  spiWP         = 2                            ' IO2 / /WP pin
  spiHOLD       = 3                            ' IO3 / /HOLD pin
  spiCSsd       = 4                            ' CS pin for micro-SD card
  spiCSflsh     = 5                            ' CS pin for 25Q32FVSFIG 4Mx8
  spiCSsram     = 6                            ' CS pin for 23LC1024 SRAM
  spiClk        = 7                            ' CLK pin

  ' ASCII codes
  CLR           = 16         
  CR            = 13         
  QUOTE         = 34


OBJ

  Term          : "Parallax Serial Terminal with DecN"
  Memory        : "SPI Memory Driver"
  sd[2]         : "fsrw2_6B"                                   ' SD card routine


Var

  byte  Buffer[64], mount, tbuf[20], wopen, close
  long  filenum


Pub Main

  ' Start a terminal server on the QuickStart USB port at 57.6 kilobaud
  Term.Start(57_600)
  waitcnt(_clkfreq + cnt)

  ' Start the Flash and SRAM driver and report results
  Term.Str(string(CLR, "Initializing SRAM and mounting flash memory:", CR))
  FlashDemo
  SRAMDemo
  SD_Demo
  Term.Str(string(CR, "Demonstration complete.", CR))


Pub FlashDemo | Blocks, Index, Erased

  if (Blocks := Memory.Start(spiClk, spiDI, spiDO, spiCSflsh, spiCSsram, spiWP, spiHOLD)) < 1  ' Added outer parens so it would give blocks a value
    ' There was no response from the correct flash device
    Term.Str(string("Flash memory is inaccesible.", CR))
    abort

  ' Display number of free blocks
  Term.Str(string(CR, "Flash memory has "))
  Term.Dec(Blocks)
  Term.Str(string(" blocks.", CR))        

  ' Display first 64 bytes of flash memory
  Term.Str(string(CR, "First 64 bytes of flash memory:"))
  Memory.ReadFlash(0, @Buffer, 64)
  TransmitBuffer

  Term.Str(string(CR, "Write ", QUOTE, "Hello World!", QUOTE, " to flash memory? [y/N]", CR))
  repeat
    case Term.CharIn
      "Y", "y":
        ' Write "Hello World!" to flash memory
        Erased~~
        repeat Index from 0 to 11
          if !Buffer[Index] & $FF
            Erased~
            quit
        ifnot Erased
          Term.Str(string(CR, "Erase flash memory sector 0? [y/N]", CR))
          repeat
            case Term.CharIn
              "Y", "y":
                Memory.EraseFlash(0)
                Erased~~
                quit
              "N", "n", CR:
                quit

        ifnot Erased
          quit

        Term.Str(string(CR, "Writing ", QUOTE, "Hello World!", QUOTE, " to flash memory.", CR))
        Memory.WriteFlash(0, string("Hello World!"), 13)
         
        ' Display first 64 bytes of flash memory
        Term.Str(string(CR, "First 64 bytes of flash memory:"))
        Memory.ReadFlash(0, @Buffer, 64)
        TransmitBuffer
        quit
      "N", "n", CR:
        quit

  Term.Str(string(CR, "Erase flash memory? [y/N]", CR))
  repeat
    case Term.CharIn
      "Y", "y":
        Memory.EraseFlash(0)
        Erased~~
        quit
      "N", "n", CR:
        quit

    ifnot Erased
      quit

  ' Display first 64 bytes of flash memory
  Term.Str(string(CR, "First 64 bytes of flash memory:"))
  Memory.ReadFlash(0, @Buffer, 64)
  TransmitBuffer


Pub SRAMDemo | Index, Random

  ' Display first 64 bytes of SRAM
  Term.Str(string(CR, "First 64 bytes of SRAM:"))
  Memory.ReadSRAM(0, @Buffer, 64)
  TransmitBuffer

  Term.Str(string(CR, "Write ", QUOTE, "Hello World!", QUOTE, " to SRAM? [y/N]", CR))
  repeat
    case Term.CharIn
      "Y", "y":
        Term.Str(string(CR, "Randomizing first 64 bytes of SRAM:"))
        Random := cnt
        repeat Index from 0 to 60 step 4
          Memory.WriteSRAM(Index, Random, -4)
          Random?
         
        ' Write "Hello World!" to SRAM
        Term.Str(string(CR, "Writing ", QUOTE, "Hello World!", QUOTE, " to SRAM.", CR))
        Memory.WriteSRAM(0, string("Hello World!"), 13)
         
        ' Display first 64 bytes of SRAM
        Term.Str(string(CR, "First 64 bytes of SRAM:"))
        Memory.ReadSRAM(0, @Buffer, 64)
        TransmitBuffer
        quit
      "N", "n", CR:
        quit


Pub SD_Demo

  Term.Str(string(CR, "Press ", QUOTE, "D", QUOTE, " for SD directory, ", QUOTE, "W", QUOTE, " to write ", QUOTE, "Hello World!", QUOTE, " to SD card or ", QUOTE, "X", QUOTE, " to exit", CR))
  repeat
    case Term.CharIn
      
      "D", "d": SD_Mount                                ' SD directory
        ifnot mount 
          quit
        SD_Directory
        SD_Unmount

      "W", "w":
        SD_Mount
        ifnot mount 
          quit
        Open_Write   
        sd.SDStr(string("Hello World!", 13))            ' Write message to file
        sd.pflush                                       ' Flush any residual data from sd buffer before closing    
        SD_Close
        SD_Unmount

      "X", "x", CR:
        quit
                       

Pub TransmitBuffer | Index

  repeat Index from 0 to 63
    ifnot Index // 16
      Term.CHAR(CR)
    Term.Hex(Buffer.byte[Index], 2)
    Term.CHAR(32)
  Term.CHAR(CR)


Pub GetFileName(FileNamePointer, FCBPointer) | Index, Index2

  longmove(FileNamePointer, FCBPointer, 3)
  ByteEndianSwap(FileNamePointer, 3)

  Index~~ {:= -1}
  repeat
    Index++
  until byte[FileNamePointer][Index] == $FF or Index == 8

  byte[FileNamePointer][Index++] := "." 
  If Index < 9
    bytemove(FileNamePointer + Index, FileNamePointer + 9, 3)

  repeat 3
    if byte[FileNamePointer][Index++] == $FF
      byte[FileNamePointer][--Index]~
      return

  byte[FileNamePointer][Index]~


Pub ByteEndianSwap(Address, Count) | Index, Temp

  repeat Index from 0 to Count - 1
    long[Address][Index] := ((long[Address][Index] & $FF_00_FF_00) >>  8) | ((long[Address][Index] & $00_FF_00_FF) <<  8)  
    long[Address][Index] := ((long[Address][Index] & $FF_FF_00_00) >> 16) | ((long[Address][Index] & $00_00_FF_FF) << 16)

Pub SD_Mount  | l

  l := \sd.mount_explicit(spiDO, spiClk, spiDI, spiCSsd)
   
  if l < 0
    term.str(string("SD card failed to mount / not present "))
    term.CHAR(CR)
    term.str(string("Insert SD card and press reset", 13))
    mount := 0
  else
    term.str(string("SD card mounted OK "))
    term.CHAR(CR)
    mount := 1
     
Pub SD_Unmount  | l

  l := \sd.unmount   
  if l < 0
    term.str(string("Failed to unmount "))
    term.CHAR(CR)
    mount := 1
  else  
    term.str(string("Card unmounted "))
    term.CHAR(CR)
    mount := 0

Pub SD_Directory | num_files                            

  num_files := 0
  sd.opendir

  term.str(string("File Name   Size", 13)) 
  
  repeat while 0 == sd.nextfile(@tbuf)
    num_files ++
    sd[ 1 ].popen(@tbuf, "r")
    term.str(@tbuf)
    repeat 12 - strsize(@tbuf)
      term.CHAR(32)
    term.decn(sd[ 1 ].get_filesize, 8)
    term.CHAR(32) 
    term.CHAR(CR)
    term.CHAR(10)

  term.str(string("Number of files: "))    
  term.dec(num_files)
  term.CHAR(CR)
  term.CHAR(10)
  sd.pclose

Pub Open_Write  | l

  repeat filenum from 0 to 99
    Make_Nfilename(filenum, @SDFile, 4)
    l := \sd.popen(@SDFile, "r")
    if l == -1
      quit
    else
      sd.pclose  

  Make_Nfilename(filenum++, @SDFile, 4)
  
  l := \sd.popen(@SDFile,"w")
  if l < 0
    term.str(@SDFile)  
    term.str(string(" failed to open "))
    term.CHAR(13)
    wopen := 0
  else
    term.str(@SDFile)
    term.str(string(" opened in write mode "))
    term.CHAR(13)
    wopen := 1

Pub SD_Close  | l
      
  l := \sd.pclose
  if l < 0
    term.CHAR(13)
    term.str(@SDFile)
    term.str(string(" failed to close "))
    term.CHAR(13)
    close := 0 
  else
    term.CHAR(13)
    term.str(@SDFile)
    term.str(string(" closed "))
    term.CHAR(13)
    close := 1

Pub Make_Nfilename(n, spntr, pos)

'' Adds 2 digit # n to string at spntr
'' -- pos is position of 10's digit in string

  n := 0 #> n <# 99

  byte[spntr][pos+0] := n / 10 + "0"
  byte[spntr][pos+1] := n // 10 + "0"

Dat

SDFile  byte  "TEST00.TXT", 0
             